/*
   assh.c -- Anonymous Secure SHell

   Copyright (C) 2008 jlzz <jlzz@users.sourceforge.net>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   $Id: assh.c,v 1.25 2008/08/14 13:32:36 lorenzzz Exp $
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>
#include <libgen.h>

#include "config.h"

void usage(char *);
int set_config_env(char *);
int nb_days_old(char *);
int get_random_proxy(char *,struct proxy *);
int get_random(int);

/* usage(): display help and exit program with return code equal to 1  */
void usage(char *argv0) {
	 printf("usage: %s <-i|--init> [filename]|<ssh options>\n\n\
   -h|--help             Print usage\n\
   -i|--init [filename]  Initialise and check proxy list\n\
   <ssh options>         Classical OpenSSH options, see \"ssh --help\"\n\n\
",argv0);
	 exit(1);
}


/* set_config_env(): This will create dir_name directory if it doesn't
   exists. This directory will be used to store configuration file and
   proxy list. return 0 for success or of dir_name already exist, 1 if it
   failed. */
int set_config_env(char *dir_name) {
	 struct stat dir;
	 char *home,*path;

	 path=(char *)malloc(strlen(CONFIG_DIR)+1);
	 strcpy(path,CONFIG_DIR);

	 if(strncmp(path,"~",1)==0) {
	 	 /* substitute "~" by "$HOME" */
		 home=getenv("HOME");
		 if((strlen(home)+strlen(path)+1)>MAX_PATH_LEN) {
		 	 /* path cant be stored in dir_name */
			 printf("[-] Pathname is too long\n");
			 return 1;
		 }
		 sprintf(dir_name,"%s%s",home,path+1);
	 } else {
		 strcpy(dir_name,path);
	 }

	 free(path);

	 if(stat(dir_name,&dir)==0) {
		 /* file exists */
		 if(!S_ISDIR(dir.st_mode)) {
			/* it's not a directory */
			if(unlink(dir_name)==-1) return 1;
			printf("[+] Setting up user configuration in %s\n"
			       ,dir_name);
		 	if(mkdir(dir_name,0755)==-1) return 1;
		 }

	 } else { /* file does not exist */
		 printf("[+] Setting up user configuration in %s\n",dir_name);
		 if(mkdir(dir_name,0755)==-1) return 1;
	 }

	 return 0;
}


/* nb_days_old(): return the number of days since filename as been
   modified or return -1 if file does not exist */
int nb_days_old(char *filename){
	 struct stat f;
	 time_t now;

	 time(&now);
	 if(stat(filename,&f)==-1) return -1;

	 return (now-f.st_mtime)/86400;
}


/* get_random_proxy(): get random line in filename and fill the p
   structure. return 0 is success, 1 if failed (filename empty) */
int get_random_proxy(char *filename,struct proxy *p) {
	 FILE *fp;
	 char line[100];
	 int nblines=0;
	 int rand_nb;
	 char *ptr;

	 if((fp=fopen(filename,"r"))==NULL) {
		 printf("[-] Can't open %s\n",filename);
		 return 1;
	 }

	 while(fgets(line,sizeof(line),fp)){
		 nblines++;
	 } rewind(fp);

	 if(nblines==0) return 1;

	 rand_nb=get_random(nblines);
	 for(nblines=1;nblines<=rand_nb;nblines++){
		 fgets(line,sizeof(line),fp);
         }

	 *(strchr(line,'\n'))='\0';
	 ptr=strchr(line,':');
	 strcpy(p->port,ptr+1);
	 *ptr='\0';
	 strcpy(p->ip,line);

	 return 0;
}


/* return a random number between 1 and max_value */
int get_random(int max_value){
        srand(time(0));
        return rand()%max_value+1;
}


int main(int argc, char **argv){
	 int init_mode=0;        /* Initialisation mode flag */
	 char *config_dir;       /* Configuration path */
	 char *config_file;      /* Configuration file */
	 int nb_days;            /* The age of configuration file */
	 char opt[200];          /* Options of the ssh command */
	 char cmd[300];          /* Final command */
	 struct proxy p;	 /* Proxy structure (ip,port) */
	 char *fullpath;
	 char *srcfile;
	 FILE *final;
	 char line[100];
	 int nb_line=0;

	 printf("Anonymous Secure SHell (Version %s)\n",VERSION);

	 if((argc == 1) || !strcmp(argv[1],"-h") || !strcmp(argv[1],"--help")) usage(argv[0]);

	 if(!strcmp(argv[1],"-i") || !strcmp(argv[1],"--init")) {
		 init_mode=1;
		 srcfile=argv[2];
	 }

	 config_dir=(char *)malloc(MAX_PATH_LEN);
	 if(set_config_env(config_dir)!=0) {
		 printf("[-] Setting configuration environment failed\n");
		 return 1;
	 } /* config_dir is now set ! */
	 config_file=malloc(strlen(config_dir)+strlen(CONFIG_FILE)+1);
	 sprintf(config_file,"%s/%s",config_dir,CONFIG_FILE);


	 if(init_mode==1) {
		 if(strcmp(SSH_SERVER_DOMAIN,"hostname")==0) {
			 printf(
"[-] Initialisation mode needs a ssh server to connect to, and it \n"
"    seems you did not specified it. Please modify config.h at line 28\n"
"    and recompile\n");
			 exit(1);
		 }
		 
		 if(srcfile == NULL) { /* no proxy file in argument, check URL */
		 	 printf("[+] Initialisation from url (%s)\n",PROXY_URL);
		 	 get_proxy_list();
		 	 printf("[+] Proxies list creation (hold on %d sec.)\n",CHECK_TIMEOUT);
		 	 write_proxy_list(config_file);
		 } else { /* proxies are in srcfile */
			 printf("[+] Initialisation from file (%s)\n",srcfile);
			 printf("[+] Proxies list creation (hold on %d sec.)\n",CHECK_TIMEOUT);
			 gen_proxy_list_from_file(srcfile,config_file);
		 }
		 printf("[+] Initialisation finished, ");
		 final=fopen(config_file,"r");
		 while(fgets(line,sizeof(line),final)!=NULL){
			 nb_line++;
		 }
		 fclose(final);
		 if(nb_line==0) {
			printf("no proxy found, find out another list to check\n");
		 } else printf("%d proxies found. Enjoy!\n",nb_line);
	 } else {

		 nb_days=nb_days_old(config_file);
		 if(nb_days>MAX_DAYS_OLD || nb_days==-1) {
		         /* configuration file is too old */
			 printf("[-] %s is older than %d days or does not exist.\n"
			        ,config_file,MAX_DAYS_OLD);
			 printf("[!] Initialize assh first (option --init)\n");
			 return 1;
		 }

		 if(get_random_proxy(config_file,&p)==1) {
			 printf("[-] Can't get random proxy, file empty ?\n");
			 return 1;
		 }

		 if((*argv[0] != '.') && (*argv[0] != '/')) {
			 fullpath=malloc(strlen("/usr/bin")+1);
			 strcpy(fullpath,"/usr/bin");
		 } else
			fullpath=dirname(argv[0]);

		 memset(opt,0,sizeof(opt));
		 while(--argc>0) {
			argv++;
			strncat(opt,*argv,strlen(*argv));
			strncat(opt," ",1);
		 }

		 sprintf(cmd,"/usr/bin/ssh \
-o \"ProxyCommand %s/corkscrew %s %s %%h %%p \" %s",fullpath,p.ip,p.port,opt);
		 printf("[+] Connection (using proxy %s:%s)...\n",p.ip,p.port);
		 if(DEBUG)printf("DEBUG: %s\n",cmd);

		 system(cmd);
	 }

	 free(config_dir);
	 free(config_file);

	 return 0;
}
